@@ -12,8 +12,13 @@
 //#define AVOID_INPUT_DIALOG
 
 MainWindow::MainWindow()
-    : eqEffect (0), plman (this), settings (tr ("TomAmp"), "TomAmp"), isPlaying (false), autostopped (false)
+    : eqEffect (0),
+    plman (this), 
+    settings (tr ("TomAmp"), "TomAmp"),
+    isPlaying (false),
+    autostopped (false)
 {
+    mwa = new MainWindowActions (this, settings.value("repeat", false).toBool (), settings.value("shuffle", false).toBool ());
     QList<Phonon::EffectDescription> effectDescriptions =
                  Phonon::BackendCapabilities::availableAudioEffects();
     audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
@@ -92,7 +97,10 @@ MainWindow::MainWindow()
     setWindowTitle("TomAmp");
     show ();
     timeLcd->display("00:00:00");
-    plman.addStringList(settings.value("lastPlaylist").toStringList());
+    plman.addStringList(settings.value("lastPlaylist").toStringList(),
+                        settings.value("playlistTitles").toStringList(),
+                        settings.value("playlistPositions").toList()
+                        );
     setupShuffleList();
     int curind = settings.value("currentIndex", -1).toInt ();
     if (curind >= 0)
@@ -107,6 +115,8 @@ MainWindow::~MainWindow()
     settings.setValue("shuffle", shuffle);
     settings.setValue("repeat", repeat);
     settings.setValue("lastPlaylist", plman.playlistStrings());
+    settings.setValue("playlistTitles", plman.playlistTitles());
+    settings.setValue("playlistPositions", plman.playlistPositions());
     settings.setValue("volume", audioOutput->volume());
     settings.setValue("currentIndex", lastPlayed);
     settings.setValue("headers", headers);
@@ -158,7 +168,7 @@ void MainWindow::addFiles()
     if (folder.isEmpty())
         folder = QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
     QString ext = "*." + plman.allowedExt().join(" *.");
-    ext = "Music files (" + ext + ");;Playlists (*.m3u *.pls)";
+    ext = "Music files (" + ext + ");;Playlists (*.m3u *.pls *.cue)";
     QStringList files = QFileDialog::getOpenFileNames(this, tr("Select Files To Add"),
                                                       folder, ext);
 
@@ -170,7 +180,7 @@ void MainWindow::addFiles()
     QStringList toadd;
     foreach (QString string, files)
     {
-        if (string.toLower().endsWith(".pls") || string.toLower().endsWith(".m3u"))
+        if (string.toLower().endsWith(".pls") || string.toLower().endsWith(".m3u") || string.toLower().endsWith(".cue"))
             plman.addPlaylist(string);
         else
             toadd.append (string);
@@ -223,7 +233,7 @@ void MainWindow::about()
     QMessageBox msg;
     msg.setIconPixmap(QPixmap (":images/Tomamp.png"));
     msg.setModal(true);
-    msg.setWindowTitle(tr ("About TomAmp v0.4"));
+    msg.setWindowTitle(tr ("About TomAmp v0.4.2"));
     msg.setText(tr("TomAmp is a simple playlist-based music player.\n\n"
                 "(c) 2010 Tamas Marki <tmarki@gmail.com>\n"
                 "Please send comments and bug reports to the above e-mail address.\n\n"
@@ -234,6 +244,7 @@ void MainWindow::about()
 
 void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState */)
 {
+    int ind = plman.indexOf(mediaObject->currentSource());
     switch (newState)
     {
         case Phonon::ErrorState:
@@ -250,44 +261,28 @@ void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState
             next ();
             break;
         case Phonon::PlayingState:
+            mwa->playingState();
             setWindowTitle(mediaObject->metaData().value("TITLE") + "(" + mediaObject->metaData().value("ARTIST") + ") - TomAmp");
-            if (stopAction->isVisible())
-            {            
-                pauseAction->setVisible(true);
-                playAction->setVisible (false);
-            }
-            playAction->setEnabled(false);
-            pauseAction->setEnabled(true);
-            stopAction->setEnabled(true);
             unhighlightRow(lastPlayed);
             lastPlayed = plman.indexOf(mediaObject->currentSource());
             highlightRow(plman.indexOf(mediaObject->currentSource()));
             musicTable->selectRow(plman.indexOf(mediaObject->currentSource()));
             autostopped = false;
+            if (plman.getItem(ind).subPos)
+            {
+                qDebug () << "subpos " << plman.getItem(ind).subPos;
+                mediaObject->seek(plman.getItem(ind).subPos);
+            }
             break;
         case Phonon::StoppedState:
+            mwa->stoppedState();
             setWindowTitle("TomAmp");
-            stopAction->setEnabled(false);
-            playAction->setEnabled(true);
-            if (stopAction->isVisible())
-            {            
-                pauseAction->setVisible(false);
-                playAction->setVisible(true);
-            }
-            pauseAction->setEnabled(false);
             timeLcd->display("00:00:00");
             unhighlightRow(plman.indexOf(mediaObject->currentSource()));
             autostopped = false;
             break;
         case Phonon::PausedState:
-            pauseAction->setEnabled(false);
-            stopAction->setEnabled(true);
-            if (stopAction->isVisible())
-            {            
-                pauseAction->setVisible(false);
-                playAction->setVisible(true);
-            }
-            playAction->setEnabled(true);
+            mwa->pausedState();
             break;
         case Phonon::BufferingState:
             break;
@@ -554,100 +549,6 @@ void MainWindow::finished()
 
 void MainWindow::setupActions()
 {
-    playAction = new QAction(QIcon (QPixmap (":images/play.png")), "", this);
-    playAction->setShortcut(tr("Crl+P"));
-    playAction->setDisabled(true);
-    pauseAction = new QAction(QIcon (QPixmap (":images/pause.png")), "", this);
-    pauseAction->setShortcut(tr("Ctrl+A"));
-    pauseAction->setDisabled(true);
-    pauseAction->setVisible(false);
-    stopAction = new QAction(QIcon (QPixmap (":images/stop.png")), "", this);
-    stopAction->setShortcut(tr("Ctrl+S"));
-    stopAction->setDisabled(true);
-    nextAction = new QAction(QIcon (QPixmap (":images/next.png")), "", this);
-    nextAction->setShortcut(tr("Ctrl+N"));
-//    upAction = new QAction (QString::fromUtf8("▲"), this);
-//    downAction = new QAction (QString::fromUtf8("▼"), this);
-//    delAction = new QAction (QString::fromUtf8("╳"), this);
-//    enqueueActionButton = new QAction (tr ("E"), this);
-    upAction = new QAction(QIcon (QPixmap (":images/up.png")), "", this);
-    upAction->setVisible(false);
-    downAction = new QAction(QIcon (QPixmap (":images/down.png")), "", this);
-    downAction->setVisible(false);
-    delAction = new QAction(QIcon (QPixmap (":images/remove.png")), "", this);
-    delAction->setVisible(false);
-    addAction = new QAction(QIcon (QPixmap (":images/add.png")), "", this);
-    addAction->setVisible(false);
-    enqueueActionButton = new QAction(QIcon (QPixmap (":images/enqueue.png")), "", this);
-    enqueueActionButton->setVisible(false);
-    previousAction = new QAction(QIcon (QPixmap (":images/previous.png")), "", this);
-    previousAction->setShortcut(tr("Ctrl+R"));
-    if (repeat)
-        repeatAction = new QAction(QIcon (QPixmap (":images/repeat.png")), "", this);
-    else
-        repeatAction = new QAction(QIcon (QPixmap (":images/repeat_disabled.png")), "", this);
-    repeatAction->setShortcut(tr("Ctrl+I"));
-    if (shuffle)
-        shuffleAction = new QAction(QIcon (QPixmap (":images/shuffle.png")), "", this);
-    else
-        shuffleAction = new QAction(QIcon (QPixmap (":images/shuffle_disabled.png")), "", this);
-    shuffleAction->setShortcut(tr("Ctrl+H"));
-    volumeAction = new QAction(QIcon (QPixmap (":images/volume.png")), "", this);
-    volumeAction->setVisible(false);
-    volumeAction->setCheckable(true);
-    toggleControls = new QAction(QIcon (QPixmap (":images/arrow1.png")), "", this);
-    addFilesAction = new QAction(tr("Add &File"), this);
-    addFilesAction->setShortcut(tr("Ctrl+F"));
-    addFoldersAction = new QAction(tr("Add F&older"), this);
-    addFoldersAction->setShortcut(tr("Ctrl+O"));
-    addUrlAction = new QAction(tr("Add &Url"), this);
-    addUrlAction->setShortcut(tr("Ctrl+U"));
-    savePlaylistAction = new QAction (tr("Sa&ve Playlist"), this);
-    savePlaylistAction->setShortcut(tr ("Ctrl+V"));
-    loadPlaylistAction = new QAction (tr("&Load Playlist"), this);
-    loadPlaylistAction->setShortcut(tr("Ctrl+L"));
-    clearPlaylistAction = new QAction (tr("&Clear Playlist"), this);
-    clearPlaylistAction->setShortcut(tr("Ctrl+C"));
-    optionAction = new QAction (tr("Op&tions"), this);
-    optionAction->setShortcut(tr("Ctrl+T"));
-    equalizerAction = new QAction (tr("&Equalizer"), this);
-    equalizerAction->setShortcut(tr("Ctrl+E"));
-    exitAction = new QAction(tr("E&xit"), this);
-    exitAction->setShortcut(tr("Ctrl+X"));
-    aboutAction = new QAction(tr("A&bout"), this);
-    aboutAction->setShortcut(tr("Ctrl+B"));
-    aboutQtAction = new QAction(tr("About &Qt"), this);
-    aboutQtAction->setShortcut(tr("Ctrl+Q"));
-/*    removeSelected = new QAction (tr("&Delete from playlist"));
-    removeSelected->setShortcut(tr ("Ctrl+D"));*/
-
-    connect(playAction, SIGNAL(triggered()), this, SLOT(play()));
-    connect(pauseAction, SIGNAL(triggered()), mediaObject, SLOT(pause()) );
-    connect(stopAction, SIGNAL(triggered()), this, SLOT(stop()));
-    connect(repeatAction, SIGNAL(triggered()), this, SLOT(repeatToggle()));
-    connect(shuffleAction, SIGNAL(triggered()), this, SLOT(shuffleToggle()));
-    connect(volumeAction, SIGNAL(triggered()), this, SLOT(volumeToggle()));
-    connect(enqueueActionButton, SIGNAL(triggered()), this, SLOT(enqueueSelected()));
-    connect (addAction, SIGNAL(triggered()), this, SLOT(showAddMenu()));
-    connect(toggleControls, SIGNAL(triggered()), this, SLOT(controlsToggled()));
-
-    connect(addFilesAction, SIGNAL(triggered()), this, SLOT(addFiles()));
-    connect(addFoldersAction, SIGNAL(triggered()), this, SLOT(addFolder()));
-    connect(addUrlAction, SIGNAL(triggered()), this, SLOT(addUrl()));
-    connect (savePlaylistAction, SIGNAL (triggered()), this, SLOT (savePlaylist()));
-    connect (loadPlaylistAction, SIGNAL (triggered()), this, SLOT (loadPlaylist()));
-    connect (clearPlaylistAction, SIGNAL (triggered()), &plman, SLOT (clearPlaylist()));
-    connect (optionAction, SIGNAL (triggered()), this, SLOT (showOptions()));
-    connect (equalizerAction, SIGNAL (triggered()), this, SLOT (showEqualizer()));
-    connect (nextAction, SIGNAL(triggered()), this, SLOT(next()));
-    connect (previousAction, SIGNAL(triggered()), this, SLOT(previous()));
-    connect (upAction, SIGNAL(triggered()), this, SLOT(upSelected()));
-    connect (downAction, SIGNAL(triggered()), this, SLOT(downSelected()));
-    connect (delAction, SIGNAL(triggered()), this, SLOT(removeSelectedItem()));
-    connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));
-    connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));
-    connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
-//    connect (removeSelected, SIGNAL (triggered()), this, SLOT (removeSelectedItem()));
 }
 
 void MainWindow::removeSelectedItem()
@@ -678,26 +579,18 @@ void MainWindow::repeatToggle ()
 {
     repeat = !repeat;
     settings.setValue("repeat", QVariant (repeat));
-    if (repeat)
-        repeatAction->setIcon(QIcon (QPixmap (":images/repeat.png")));
-    else
-        repeatAction->setIcon(QIcon (QPixmap (":images/repeat_disabled.png")));
+    mwa->repeatToggle(repeat);
 }
 
 void MainWindow::shuffleToggle ()
 {
     shuffle = !shuffle;
     settings.setValue("shuffle", QVariant (shuffle));
-    if (shuffle)
-        shuffleAction->setIcon(QIcon (QPixmap (":images/shuffle.png")));
-    else
-        shuffleAction->setIcon(QIcon (QPixmap (":images/shuffle_disabled.png")));
 }
 
 void MainWindow::volumeToggle ()
 {
-    volumeSlider->setVisible(volumeAction->isChecked());
-    musicTable->show();
+    volumeSlider->setVisible(mwa->isVolumeChecked());
     timer.start(5000);
 }
 
@@ -705,7 +598,7 @@ void MainWindow::hideVolumeBar()
 {
     volumeSlider->hide();
     musicTable->show();
-    volumeAction->setChecked(false);
+    mwa->setVolumeCheck(false);
 }
 
 void MainWindow::play()
@@ -725,21 +618,9 @@ void MainWindow::stop()
 
 void MainWindow::setupMenus()
 {
-    QMenu *fileMenu = menuBar()->addMenu(tr("&File"));
-    fileMenu->addAction(addFilesAction);
-    fileMenu->addAction(addFoldersAction);
-    fileMenu->addAction(addUrlAction);
-    fileMenu->addSeparator();
-    fileMenu->addAction(savePlaylistAction);
-    fileMenu->addAction(loadPlaylistAction);
-    fileMenu->addAction(clearPlaylistAction);
-    fileMenu->addAction(optionAction);
-    fileMenu->addAction(equalizerAction);
-//    fileMenu->addAction(exitAction);
-
-    QMenu *aboutMenu = menuBar()->addMenu(tr("&Help"));
-    aboutMenu->addAction(aboutAction);
-    aboutMenu->addAction(aboutQtAction);
+    QMenu* fm = menuBar()->addMenu(tr("&File"));
+    QMenu* hm = menuBar()->addMenu(tr("&Help"));
+    mwa->setupMenus(fm, hm);
 }
 
 void MainWindow::setupUi()
@@ -768,36 +649,10 @@ void MainWindow::setupUi()
     volumeSlider->setMuteVisible(false);
     int iconsize = settings.value ("iconsize", 48).toInt();
     bar->setIconSize(QSize (iconsize, iconsize));
-    bar->addAction(previousAction);
-    bar->addAction(playAction);
-    bar->addAction(pauseAction);
-    bar->addAction(stopAction);
-    bar->addAction(nextAction);
-    bar->addAction(repeatAction);
-    bar->addAction(shuffleAction);
-    bar->addAction(upAction);
-    bar->addAction(downAction);
-    bar->addAction(delAction);
-    bar->addAction(addAction);
-    bar->addAction(enqueueActionButton);
-    bar->addAction(volumeAction);
-    bar->addAction(toggleControls);
-
+    mwa->setupToolbar(bar);
     contextMenu = new QMenu (this);
-    enqueueAction = contextMenu->addAction(tr ("Enqueue"));
-    removeSelected = contextMenu->addAction(tr ("Remove selected"));
-    removeAllButSelected = contextMenu->addAction(tr("Remove all but selected"));
-    connect (removeSelected, SIGNAL (triggered()), this, SLOT (removeSelectedItem()));
-    connect (removeAllButSelected, SIGNAL (triggered()), this, SLOT (removeAllButSelectedItem()));
-    connect (enqueueAction, SIGNAL (triggered()), this, SLOT (enqueueSelected()));
 
     addMenu = new QMenu (this);
-    addMenuAddFile = addMenu->addAction(tr ("Add File"));
-    addMenuAddUrl = addMenu->addAction(tr ("Add Url"));
-    addMenuAddFolder = addMenu->addAction(tr ("Add Folder"));
-    connect (addMenuAddFile, SIGNAL(triggered()), this, SLOT(addFiles()));
-    connect (addMenuAddUrl, SIGNAL(triggered()), this, SLOT(addUrl()));
-    connect (addMenuAddFolder, SIGNAL(triggered()), this, SLOT(addFolder()));
     
 
     timeLcd = new QLCDNumber;
@@ -923,9 +778,11 @@ void MainWindow::setupShuffleList()
 
 void MainWindow::savePlaylist ()
 {
-    QString filename = QFileDialog::getSaveFileName(this, tr("Please select file name"), "", "Playlist Files (*.m3u *.pls)");
+    QString filename = QFileDialog::getSaveFileName(this, tr("Please select file name"), settings.value ("LastFolder", "").toString(), "Playlist Files (*.m3u *.pls)");
     if (filename.isEmpty())
         return;
+    QString dir = QFileInfo (filename).absoluteDir().absolutePath();
+    settings.setValue("LastFolder", dir);
     if (!plman.savePlaylist(filename))
     {
 #ifdef Q_WS_MAEMO_5
@@ -940,9 +797,11 @@ void MainWindow::savePlaylist ()
 
 void MainWindow::loadPlaylist ()
 {
-    QString filename = QFileDialog::getOpenFileName(this, tr("Select playlist file to load"), "", "*.m3u *.pls");
+    QString filename = QFileDialog::getOpenFileName(this, tr("Select playlist file to load"), settings.value ("LastFolder", "").toString(), "*.m3u *.pls *.cue");
     if (filename.isEmpty())
         return;
+    QString dir = QFileInfo (filename).absoluteDir().absolutePath();
+    settings.setValue("LastFolder", dir);
     plman.loadPlaylist (filename);
 }
 
@@ -1169,8 +1028,10 @@ void MainWindow::showOptions ()
     OptionDialog* dlg = new OptionDialog (this, settings);
     dlg->exec();
     delete dlg;
+    qDebug () << "Options executed";
     if (oldiconsize != settings.value("iconsize", 48).toInt())
     {
+        qDebug () << "Iconsize change, gui redraw";
         delete centralWidget();
         setupUi();
     }
@@ -1179,6 +1040,7 @@ void MainWindow::showOptions ()
         || oldsize != settings.value("rowHeight", DEFAULT_ROW_HEIGHT).toInt()
         || oldiconsize != settings.value("iconsize", 48).toInt())
     {
+        qDebug () << "Headers, row height or iconsize change: " << headers << " != " << settings.value("headers", QStringList ()).toStringList() << " or " << oldsize << " != " << settings.value("rowHeight", DEFAULT_ROW_HEIGHT).toInt() << " or " << oldiconsize << " != " << settings.value("iconsize", 48).toInt();
         headers = settings.value("headers", QStringList ()).toStringList();
         musicTable->setColumnCount(headers.size ());
         musicTable->setHorizontalHeaderLabels(headers);
@@ -1239,12 +1101,12 @@ void MainWindow::headphoneEvent(QString what,QString how)
 
     if (mediaObject->state() == Phonon::PlayingState && !present)
     {
-        pauseAction->trigger();
+        mwa->triggerPause();
         autostopped = true;
     }
     else if (mediaObject->state() == Phonon::PausedState && autostopped && present)
     {
-        playAction->trigger();
+        mwa->triggerPlay();
         autostopped = false;
     }
 }
@@ -1262,40 +1124,6 @@ void MainWindow::equalizerChanged (int ind, int val)
 
 void MainWindow::controlsToggled ()
 {
-    if (stopAction->isVisible())
-    {
-        playAction->setVisible(false);
-        pauseAction->setVisible(false);
-        stopAction->setVisible(false);
-        nextAction->setVisible(false);
-        previousAction->setVisible(false);
-        shuffleAction->setVisible(false);
-        repeatAction->setVisible(false);
-        upAction->setVisible(true);
-        downAction->setVisible(true);
-        delAction->setVisible(true);
-        addAction->setVisible(true);
-        enqueueActionButton->setVisible(true);
-        volumeAction->setVisible(true);
-        toggleControls->setIcon(QIcon (QPixmap (":images/arrow2.png")));
-    }
-    else
-    {
-        if (mediaObject->state() == Phonon::PlayingState)
-            pauseAction->setVisible(true);
-        else
-            playAction->setVisible(true);
-        stopAction->setVisible(true);
-        nextAction->setVisible(true);
-        previousAction->setVisible(true);
-        shuffleAction->setVisible(true);
-        repeatAction->setVisible(true);
-        upAction->setVisible(false);
-        downAction->setVisible(false);
-        delAction->setVisible(false);
-        addAction->setVisible(false);
-        enqueueActionButton->setVisible(false);
-        volumeAction->setVisible(false);
-        toggleControls->setIcon(QIcon (QPixmap (":images/arrow1.png")));
-    }
+    mwa->controlsToggled(mediaObject->state() == Phonon::PlayingState);
 }
+
