@@ -27,8 +27,10 @@ private:
     void    setupUi ();
     QSettings& settings;
     QComboBox* orient;
+    QComboBox* buttonSize;
     QComboBox* rowHeight;
     QCheckBox* hpsense;
+    QList<QCheckBox*> headerCbs;
     QStringList availableHeaders;
     QVBoxLayout *headerLayout;
 };
